"""
Useful functions for generating data to test neural networks
"""
import numpy as np
import torch

# Copyright (c) 2015 Andrej Karpathy
# License: https://github.com/cs231n/cs231n.github.io/blob/master/LICENSE
# Source: https://cs231n.github.io/neural-networks-case-study/
def spiral_datagen(samples: int, classes: int) -> tuple[torch.Tensor, torch.Tensor]:
    """
    Returns X and y vectors with specified number of samples * classes
    
    Usage: 
        x, y = spiral_datagen(200, 2) # 400 samples with 2 classes

    To print with pyplot use:
        colors = ['blue', 'green']
        for i in zip(x, y):
            plt.plot(i[0][0].item(), i[0][1].item(), 'ro', color=colors[i[1].item()])
        plt.show()
    """
    X = np.zeros((samples*classes, 2))
    y = np.zeros(samples*classes, dtype='uint8')
    for class_number in range(classes):
        ix = range(samples*class_number, samples*(class_number+1))
        r = np.linspace(0.0, 1, samples)
        t = np.linspace(class_number*4, (class_number+1)*4, samples) + np.random.randn(samples)*0.2
        X[ix] = np.c_[r*np.sin(t*2.5), r*np.cos(t*2.5)]
        y[ix] = class_number
    X = torch.from_numpy(np.float32(X))
    y = torch.from_numpy(y)
    return X, y


import torch
import matplotlib.pyplot as plt
from torch.utils.data import Dataset


def one_hot_encode(position: int, num_classes: int):
    """
    Returns a one hot encoded 1d row tensor: Tensor([1, num_classes])

    Usage:
        # this will one hot encode a 1d vector with 5 classes
        # the 'number one' will be placed at index 0
        
        > one_hot_encode(0, 5)
        [1, 0, 0, 0, 0]
    """
    zero_vector = torch.zeros(num_classes, dtype=torch.float32) 
    zero_vector[position] = 1.
    return zero_vector

def plot_image_dataset(dataset: Dataset, n_items: int=1, cmap='gray') -> None:
    """
    Plot 1 or more images
    dataset: torch Dataset object containing X and Y matrices
    n_items: Number of items to display in the plot. Default is 1
    """
    assert isinstance(dataset, Dataset), "Dataset provided is not a torch Dataset class"
    assert len(dataset) >= n_items, "number of items is greater than dataset length"
    fig = plt.figure(figsize=(8, 8))

    for i in range(n_items):
        img = dataset[i][0][0]
        fig.add_subplot(5, 4, i+1)
        plt.imshow(img, cmap=cmap)
    plt.show()
